unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Registry, ShlObj, ActiveX, Mask, ComCtrls;

type
  TMainForm = class(TForm)
    GroupBox1: TGroupBox;
    Label1: TLabel;
    edtComputerName: TEdit;
    btnBrowse: TButton;
    GroupBox2: TGroupBox;
    chkForce: TCheckBox;
    chkReboot: TCheckBox;
    btnOK: TButton;
    btnAbort: TButton;
    btnExit: TButton;
    Label2: TLabel;
    Memo: TMemo;
    edtTimeout: TEdit;
    Label3: TLabel;
    udTimeout: TUpDown;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnBrowseClick(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure btnAbortClick(Sender: TObject);
    procedure btnExitClick(Sender: TObject);
  private
    { Private declarations }
    FReg: TRegistry;
    FLocalComputerName: string;
    procedure EnablePrivileges;
    procedure DisablePrivileges;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

const
  INIT_KEY                = 'Software\NetUtils\Rshdown\';
  SE_SHUTDOWN_NAME        = 'SeShutdownPrivilege';
  SE_REMOTE_SHUTDOWN_NAME = 'SeRemoteShutdownPrivilege';

resourcestring
  STop     = 'Top';
  SLeft    = 'Left';
  SForce   = 'Force';
  SReboot  = 'Reboot';
  STimeout = 'Timeout';

var
  hToken: THandle;
  tkp, tkp_prev: TTokenPrivileges;
  PrivilName: PChar;
  ReturnLength: Cardinal;

function SelectComputer(const Caption: string; out Computer: string): Boolean;
var
   BrowseInfo: TBrowseInfo;
   ShellMalloc: IMalloc;
   RootItemIDList, ItemIDList: PItemIDList;
   Buffer: PChar;
begin
     Result := False;
     Computer := '';
     FillChar(BrowseInfo, SizeOf(BrowseInfo), 0);
     if (ShGetMalloc(ShellMalloc) = S_OK) and (ShellMalloc <> nil) then
     begin
          Buffer := ShellMalloc.Alloc(MAX_COMPUTERNAME_LENGTH + 1);
          try
             SHGetSpecialFolderLocation(0, CSIDL_NETWORK, RootItemIDList);
             with BrowseInfo do
             begin
                  hwndOwner := Application.Handle;
                  pidlRoot := RootItemIDList;
                  pszDisplayName := Buffer;
                  lpszTitle := PChar(Caption);
                  ulFlags := BIF_BROWSEFORCOMPUTER;
             end;
             ItemIDList := ShBrowseForFolder(BrowseInfo);
             Result := ItemIDList <> nil;
             if Result then
             begin
                  ShellMalloc.Free(ItemIDList);
                  Computer := '\\' + AnsiUpperCase(Buffer);
             end;
          finally
             ShellMalloc.Free(RootItemIDList);
             ShellMalloc.Free(Buffer);
          end;
     end;
end;

procedure TMainForm.EnablePrivileges;
begin
     if not OpenProcessToken(GetCurrentProcess, TOKEN_ADJUST_PRIVILEGES or TOKEN_QUERY, hToken) then
        raise Exception.Create('OpenProcessToken failed.');
     if AnsiUpperCase(edtComputerName.Text) = AnsiUpperCase(FLocalComputerName) then
        PrivilName := SE_SHUTDOWN_NAME
     else
        PrivilName := SE_REMOTE_SHUTDOWN_NAME;
     if not LookupPrivilegeValue(PChar(edtComputerName.Text), PrivilName, tkp.Privileges[0].Luid) then
        raise Exception.Create('You do not have Remote Shutdown Privilege on this'#13#10 +
                                'computer, or computer is not a Windows NT computer,'#13#10 +
                                'or computer cannot be found on the network.');
     tkp.PrivilegeCount := 1;
     tkp.Privileges[0].Attributes := SE_PRIVILEGE_ENABLED;
     AdjustTokenPrivileges(hToken, False, tkp, SizeOf(TTokenPrivileges), tkp_prev, ReturnLength);
     if GetLastError <> ERROR_SUCCESS then
        raise Exception.Create('AdjustTokenPrivileges enable failed.');
end;

procedure TMainForm.DisablePrivileges;
begin
     tkp.Privileges[0].Attributes := 0;
     AdjustTokenPrivileges(hToken, False, tkp, SizeOf(TTokenPrivileges), tkp_prev, ReturnLength);
     if GetLastError <> ERROR_SUCCESS then
        raise Exception.Create('AdjustTokenPrivileges disable failed.');
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
   ComputerName: array[0..MAX_COMPUTERNAME_LENGTH - 1] of Char;
   MaxComputerName: Cardinal;
begin
     FReg := TRegistry.Create;
     with FReg do if KeyExists(INIT_KEY) then
     begin
          OpenKey(INIT_KEY, False);
          if ValueExists(STop) then Top := ReadInteger(STop);
          if ValueExists(SLeft) then Left := ReadInteger(SLeft);
          if ValueExists(SForce) then chkForce.Checked := ReadBool(SForce);
          if ValueExists(SReboot) then chkReboot.Checked := ReadBool(SReboot);
          if ValueExists(STimeout) then udTimeout.Position := ReadInteger(STimeout);
          CloseKey;
     end;
     MaxComputerName := SizeOf(ComputerName);
     GetComputerName(ComputerName, MaxComputerName);
     edtComputerName.Text := '\\' + ComputerName;
     FLocalComputerName := edtComputerName.Text;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
     with FReg do
     begin
          OpenKey(INIT_KEY, True);
          WriteInteger(STop, Top);
          WriteInteger(SLeft, Left);
          WriteBool(SForce, chkForce.Checked);
          WriteBool(SReboot, chkReboot.Checked);
          WriteInteger(STimeout, udTimeout.Position);
          CloseKey;
     end;
end;

procedure TMainForm.btnBrowseClick(Sender: TObject);
var
   s: string;
begin
     SelectComputer('Select computer', s);
     if s <> '' then edtComputerName.Text := s;
end;

procedure TMainForm.btnOKClick(Sender: TObject);
begin
     if Application.MessageBox('Do you want to continue?',
                               PChar(Application.Title),
                               MB_ICONQUESTION or MB_YESNO) = IDNO then Exit;
     EnablePrivileges;
     if not InitiateSystemShutdown(PChar(edtComputerName.Text),
                                   PChar(Memo.Text),
                                   StrToInt(Trim(edtTimeout.Text)),
                                   chkForce.Checked,
                                   chkReboot.Checked) then
        raise Exception.Create('InitiateSystemShutdown failed.');
     DisablePrivileges;
     btnAbort.Enabled := True;
end;

procedure TMainForm.btnAbortClick(Sender: TObject);
begin
     EnablePrivileges;
     if not AbortSystemShutdown(PChar(edtComputerName.Text)) then
        raise Exception.Create('AbortSystemShutdown failed.');
     DisablePrivileges;
     btnAbort.Enabled := False;
end;

procedure TMainForm.btnExitClick(Sender: TObject);
begin
     Close;
end;

end.
